package My::Bar;

sub barSub {}

1;

__END__

=head1 NAME

My::Bar - blah

=head1 SYNOPSIS

  use My::Bar;

=head1 DESCRIPTION

blah blah

=cut

=begin WSDL

	_ATTR _bar $negativeInteger _NEEDED a bar
	_ATTR _boerk $boolean a nillable _boerk

=cut

