package My::Foo;

1;

__END__

=head1 NAME

My::Foo - blah

=head1 SYNOPSIS

  use My::Foo;

=head1 DESCRIPTION

blah blah

=cut

=begin WSDL

	_ATTR _bar $negativeInteger _NEEDED a bar
	_ATTR _boerk $boolean a nillable _boerk

=cut

